#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar 13 16:41:20 2017

@author: jonleonhardsen
"""
import numpy as np
import matplotlib.pyplot as plt
import Generate_Sailingspeeds as GS
import math
import statistics as st
import pandas as pd
import BulbData as BD

def FuelCalc():
    #FuelCalc calculates fuel consumption as a function of sailing speed
    global power
    global fuelCons
    global bulbData
    BD.resistanceData() #Runs script that formats and interpolates resistance data for different bulbs
    bulbData = BD.x     #Fetch resistance data for bulb configurations
    nH = 1.2    
    n0 = 0.55
    nR = 1.00
    nB = n0*nR
    nS = 0.99
    nT = nH*nB*nS
    sea_margin = 0.2
    bulbConfigs = int((len(bulbData[0,:])+1)/2)
    power = np.zeros((len(bulbData[:,0]),bulbConfigs))
    for i in range(power.shape[0]):         #Power calculations
        for j in range(1,power.shape[1]):
            power[i,j] = (1+sea_margin)*bulbData[i,2*j-1]*(0.51444)*bulbData[i,0]/(nT*1000)
            power[i,0] = bulbData[i,0]

    plt.figure()
    for i in range(1,power.shape[1]):
        plt.plot(power[:,0], power[:,i])
    
    plt.xlabel('Speed [knots]')
    plt.ylabel('$P_B$ [kW]')
    plt.show()
        
    #Fetches data on specific fuel consumption, as presented in thesis
    xl = pd.ExcelFile("/.../.../.../.../.../.../.../specificfuel.xlsx")
    xl.sheet_names
    sfcpd = xl.parse("Ark4")
    sfc = sfcpd.as_matrix()
    
    plt.figure()
    plt.plot(sfc[:,0],sfc[:,1],c='k')
    plt.ylabel('SFC [g/kWh]')
    plt.xlabel('Speed [knots]')
    plt.show()
    fuelCons = np.zeros((len(power[:,0]),len(power[0,:])))
    
    for i in range(power.shape[0]):     #Fuel calculations
        for j in range(1,power.shape[1]):
            fuelCons[i,j] = 24*power[i,j]*sfc[i,1]/10**6
            fuelCons[i,0] = power[i,0]

    plt.figure()
    for i in range(1,power.shape[1]):
        plt.plot(fuelCons[:,0], fuelCons[:,i])
    
    plt.xlabel('Speed [knots]')
    plt.ylabel('Fuel Consumption [MT per day]')
    plt.show()

def AggFuelConsumption(speedBehaviour,speedDays):
    #Takes in sampled speed paths from SimulateDailing()
    global StatAgg
    global FlexAgg
    global FlexAgg1
    global portFlexAgg
    global bulbIDopt
    global bulbTheoretical
    global bulbActual
    global bulbModePort
    global FuelSavings
    global FuelSavings1
    global FuelSavingsPort
    distance = 0
    StatAgg = 0
    StatAgg1 = 0
    FlexAgg = 0
    FlexAgg1 = 0
    bulbIDopt = [0]*len(speedBehaviour)  
    #############################################################
    ## Fuel consumption with bulb change allowed every 2 hours ##
    #############################################################
    global triggerSpeed
    global triggerDay
    triggerDay = list()
    triggerSpeed = list()
    for i in range(0,len(speedBehaviour)):
        idx = (np.abs(fuelCons[:,0]-speedBehaviour[i])).argmin()    
        StatAgg += (2/24)*fuelCons[idx,1]               #Adds consumed fuel for original bulb
        fuelData = fuelCons[idx,1:]
        bulbIDopt[i] = (np.abs(fuelData-0)).argmin()+1  #Selects bulb configuration with minimal fuel cons
        FlexAgg += (2/24)*fuelCons[idx,bulbIDopt[i]]    #Adds consumed fuel according to bulb config
        distance += fuelCons[idx,0]*2

    FuelSavings = (StatAgg-FlexAgg)/StatAgg
    GS.bulbDistribution += bulbIDopt
    #############################################################
    ## Fuel consumption with bulb change allowed every X hours ##
    #############################################################    
    bulbTheoretical = [0]*len(speedBehaviour) #Preallocated list for optimal bulb config at each time index
    bulbActual = [0]*len(speedBehaviour) #Preallocated list for selected bulb config
    minChangeInt = 3 #Minimum allowed switching frequency, 1 = 2 hours, 2 = 4 hours, etc
    for i in range(0,len(speedBehaviour)):
        #This loop determines the mean speed determining the bulb configuration
        if i < len(speedBehaviour):
            shortTermMean = st.mean(speedBehaviour[i:i+minChangeInt])
        elif i == len(speedBehaviour)-1:
            shortTermMean = speedBehaviour[i]
       
        realIdx = (np.abs(fuelCons[:,0]-speedBehaviour[i])).argmin()
        idx = (np.abs(fuelCons[:,0]-shortTermMean)).argmin()
        fuelData = fuelCons[idx,1:]
        bulbTheoretical[i] = (np.abs(fuelData-0)).argmin()+1 #Determines optimal bulb configuration
        if i > 0 and bulbActual[i-1] != bulbTheoretical[i]:  #Determines optimal and allowed bulb configuration
            if i > minChangeInt-1 and len(set(bulbActual[i-minChangeInt:i])) == 1:
                bulbActual[i] = bulbTheoretical[i]
            else:
                bulbActual[i] = bulbActual[i-1]
        elif i == 0:
            bulbActual[i] = bulbTheoretical[i]
        else:
            bulbActual[i] = bulbActual[i-1]
        
        FlexAgg1 += (2/24)*fuelCons[realIdx,bulbActual[i]]
    FuelSavings1 = (StatAgg-FlexAgg1)/StatAgg1
    #############################################################
    ## Fuel consumption with bulb change allowed in port       ## 
    ############################################################# 
    tradeDur = 14   #Average transit duration
    trades = math.ceil(len(speedBehaviour)/((tradeDur-1)*12))
    avSpeed = [0]*trades
    steps = (tradeDur-1)*12
    for i in range(1,len(avSpeed)+1):
        #This loop calculates average speeds for each transit
        if i*steps <= len(speedBehaviour):
            avSpeed[i-1] = st.mean(speedBehaviour[(i-1)*steps+1:i*steps])
        else:
            avSpeed[i-1] = st.mean(speedBehaviour[(i-1)*steps+1:-1])   
    
    bulbOptTrade = [0]*len(avSpeed)
    for i in range(0,len(avSpeed)): #This loop selects best bulb according to transit average
        idx = (np.abs(fuelCons[:,0]-avSpeed[i])).argmin()
        fuelData = fuelCons[idx,1:]
        bulbOptTrade[i] = (np.abs(fuelData-0)).argmin()+1    
    bulbModePort = [0]*len(speedBehaviour)
    for i in range(1,len(bulbOptTrade)+1):
        if i == 1:
            for j in range((i-1)*steps,i*steps+1):
                bulbModePort[j] = bulbOptTrade[i-1]
        elif i*steps <= len(speedBehaviour):
            for j in range((i-1)*steps+1,i*steps+1):
                bulbModePort[j] = bulbOptTrade[i-1]
        else:
            for j in range((i-1)*steps+1,len(speedBehaviour)):
                bulbModePort[j] = bulbOptTrade[i-1]
    portFlexAgg = 0
    for i in range(0,len(bulbModePort)):
        idx = (np.abs(fuelCons[:,0]-speedBehaviour[i])).argmin()
        portFlexAgg += (2/24)*fuelCons[idx,bulbModePort[i]]
    FuelSavingsPort = (StatAgg-portFlexAgg)/StatAgg

if __name__ == "__main__":   
    FuelCalc()
    AggFuelConsumption()
