#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Mar  9 13:26:51 2017

@author: jonleonhardsen
"""
import math
import statistics as st
import numpy as np
import matplotlib.pyplot as plt
import Fuel_Aggregated as FA
import scipy
import BulbData as BD

def SimulateSailing(utilization,years,avtradeduration,iterations):
    #This function generates sample paths that represents sailing speed
    #utilization = Fraction of time at sea
    #years = Number of years we want to simulate
    #avtradeduration = Average transit duration
    #iterations = Number of Monte Carlo simulations
    global meanLevel
    global revRate
    global vol
    global Y
    global AverageTradeTime
    global days
    global AnY
    global savedFuel2Hours
    global savedFuelXHours
    global savedFuelPort
    global bulbDistribution
    global selected_bulbs
    global distanceSailed 
    global consumedFuelStatic
    global FOCnm
    global simdays
    meanLevel = list()
    revRate = list()
    vol = list()
    bulbDistribution = list()
    distanceSailed = list()
    consumedFuelStatic = list()
    savedFuel2Hours = [0]*iterations
    savedFuelXHours = [0]*iterations
    savedFuelPort = [0]*iterations
    simdays = list()
    for a in range(0,iterations): #For each Monte Carlo simulation
        trades = round(utilization*years*365/avtradeduration)
        dt = 2/24
        Y = [0]*12*years*365      #Pre-allocate sailing speed vector
        
        #The following parameters represents OU parameter PDFs
        vol_A = 10.75 
        vol_LOC = 0.27
        vol_SCALE = 9.65
        mean_LOC = 13.18
        mean_SCALE = 5.31
        mean_A = 0.42
        mean_C =5.22
        mean_LOC = 12.18        
        rev_K = 15.4
        rev_LOC = 0.76    
        rev_SCALE = 0.09
        
        for x in range(0,trades):
            #This loops samples OU parameters for each single transit
            meanLevel[x] = scipy.stats.gengamma.rvs(mean_A, mean_C, mean_LOC, mean_SCALE)
            revRate[x] = scipy.stats.exponnorm.rvs(rev_K, rev_LOC, rev_SCALE)
            vol[x] = scipy.stats.invgamma.rvs(vol_A, vol_LOC, vol_SCALE)
        x = 0
        portTime = round((1-utilization)*12*365/trades)     #calculates the time in port/not sailing
        MAXSPEED = 26
        for i in range(0,len(Y)):
            #This loop generates sailing speed sample paths 
            if i >= x*12*avtradeduration+portTime and i < (x+1)*12*avtradeduration + (x+1)*portTime:
                #If yes, the ship is sailing (transit)
                Y[i] = Y[i-1] + revRate[x]*(meanLevel[x]-Y[i-1])*dt + vol[x]*math.sqrt(dt)*np.random.normal(0,1)       
                if Y[i] > MAXSPEED:
                    Y[i] = MAXSPEED
            elif i >= (x+1)*12*avtradeduration + (x+1)*portTime and i < (x+1)*12*avtradeduration + (x+2)*portTime:
                #If yes, the ship is in port/not sailing
                Y[i] = 0
            elif i >= (x+1)*12*avtradeduration + (x+2)*portTime:
                #If yes, transit (including port time) is finished
                x += 1
                if x >= trades:
                    break
                else: 
                    Y[i] = meanLevel[x]
    
        days = [0]*len(Y)
        for i in range(0,len(Y)):               #Time vector
            days[i] = i/12   
        AnY = list(filter((0).__ne__, Y))       #Removes zeros (speed in port)   
        simdays = [0]*len(AnY)              
        for i in range(0,len(AnY)):             #Time vector for non-zero list
            simdays[i] = i/12

        FA.AggFuelConsumption(AnY,simdays)      #Sends sample path to fuel consum scrip/function
        savedFuel2Hours[a] = FA.FuelSavings     #Fetches saved fuel for 2hour agility (see Fuel_Aggregated for elaboration)
        savedFuelXHours[a] = FA.FuelSavings1    #Fetches saved fuel for Xhour agility (see Fuel_Aggregated for elaboration)
        savedFuelPort[a] = FA.FuelSavingsPort   #Fetches saved fuel for port agility (see Fuel_Aggregated for elaboration)
    
    
    for i in range(0,len(savedFuel2Hours)):
        savedFuel2Hours[i] = 100*savedFuel2Hours[i]
        savedFuelXHours[i] = 100*savedFuelXHours[i]
        savedFuelPort[i] = 100*savedFuelPort[i]
    
    ##################################
    #ULTIMATE AGILITY POST PROCESSING#
    ##################################
    savedFuel2Hours = [x for x in savedFuel2Hours if str(x) != 'nan']
    x = np.linspace(min(savedFuel2Hours),max(savedFuel2Hours),50)
    acc = [0]*len(x)
    for i in range(0,len(acc)):
        for j in range(0,len(savedFuel2Hours)):
            if savedFuel2Hours[j] <= x[i]:
                acc[i] += 1
    for i in range(0,len(acc)):
        acc[i] = acc[i]/len(savedFuel2Hours)
        
    std = round(st.stdev(savedFuel2Hours),5)
    mean = round(st.mean(savedFuel2Hours),5)   
    fig, ax1 = plt.subplots()
    histval1, binsval = np.histogram(savedFuel2Hours, bins=50)
    histval = histval1/sum(histval1)
    width = 0.7 * (binsval[1] - binsval[0])
    center = (binsval[:-1] + binsval[1:]) / 2
    ax1.bar(center, histval, align='center', width=width,color='k',alpha=0.5)
    ax1.set_xlabel('Saved Fuel [%]', color='k')
    ax1.set_ylabel('Distribution', color='k')
    plt.title('%s Simulations, Change Every 2nd hour \n Mean = %s, StDev = %s'%(iterations,mean,std))
    
    ax2 = ax1.twinx()
    ax2.plot(x, acc, 'k')
    ax2.set_xlabel('Saved Fuel [%]', color='k')
    ax2.set_ylabel('Probability of Missing Target', color='k')
    ax2.grid(True)
    plt.show()
    
    ##############################
    #LIMITED FLEX POST PROCESSING#
    ############################## 
    savedFuelXHours = [x for x in savedFuelXHours if str(x) != 'nan']  
    x1 = np.linspace(min(savedFuelXHours),max(savedFuelXHours),50)
    acc1 = [0]*len(x1)
    for i in range(0,len(acc1)):
        for j in range(0,len(savedFuelXHours)):
            if savedFuelXHours[j] <= x1[i]:
                acc1[i] += 1
                
    for i in range(0,len(acc1)):
        acc1[i] = acc1[i]/len(savedFuelXHours)
             
    std = round(st.stdev(savedFuelXHours),5)
    mean = round(st.mean(savedFuelXHours),5)
    fig, ax1 = plt.subplots()
    histval1, binsval = np.histogram(savedFuelXHours, bins=50)
    histval = histval1/sum(histval1)
    width = 0.7 * (binsval[1] - binsval[0])
    center = (binsval[:-1] + binsval[1:]) / 2
    ax1.bar(center, histval, align='center', width=width,color='k',alpha=0.5)
    ax1.set_xlabel('Saved Fuel [%]', color='k')
    ax1.set_ylabel('Distribution', color='k')
    plt.title('%s Simulations, Change Every 6th Hour \n Mean = %s, StDev = %s'%(iterations,mean,std))  
    ax2 = ax1.twinx()
    ax2.plot(x1, acc1, 'k')
    ax2.set_xlabel('Saved Fuel [%]', color='k')
    ax2.set_ylabel('Probability of Missing Target', color='k')
    ax2.grid(True)
    plt.show()
        
    ###########################
    #PORT FLEX POST PROCESSING#
    ###########################
    savedFuelPort = [x for x in savedFuelPort if str(x) != 'nan']   
    xP = np.linspace(min(savedFuelPort),max(savedFuelPort),50)
    accP = [0]*len(xP)
    for i in range(0,len(accP)):
        for j in range(0,len(savedFuelPort)):
            if savedFuelPort[j] <= xP[i]:
                accP[i] += 1
                
    for i in range(0,len(accP)):
        accP[i] = accP[i]/len(savedFuelPort)
        
    std = round(st.stdev(savedFuelPort),5)
    mean = round(st.mean(savedFuelPort),5)
    fig, ax1 = plt.subplots()
    histval1, binsval = np.histogram(savedFuelPort, bins=50)
    histval = histval1/sum(histval1)
    width = 0.7 * (binsval[1] - binsval[0])
    center = (binsval[:-1] + binsval[1:]) / 2
    ax1.bar(center, histval, align='center', width=width,color='k',alpha=0.5)
    ax1.set_xlabel('Saved Fuel [%]', color='k')
    ax1.set_ylabel('Distribution', color='k')
    plt.title('%s Simulations, Change In Port \n Mean = %s, StDev = %s'%(iterations,mean,std))
    ax2 = ax1.twinx()
    ax2.plot(xP, accP, 'k')
    ax2.set_xlabel('Saved Fuel [%]', color='k')
    ax2.set_ylabel('Probability of Missing Target', color='k')
    ax2.grid(True)
    plt.show()

    selected_bulbs = [0]*BD.bulbConfigs
    for i in range (0,len(selected_bulbs)):
        selected_bulbs[i] = round(bulbDistribution.count(i+1)/len(bulbDistribution),3)
            
            
if __name__ == "__main__":   
    SimulateSailing()
    
    
        
