#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Feb 22 12:57:53 2017

@author: jonleonhardsen
"""
#THIS CODE IS REFERRED TO AS THE RESAMPLING ALGORITHM AND PARAMETER ESTIMATION REGRESSION IN THE THESIS
import matplotlib
import matplotlib.pyplot as plt
matplotlib.style.use('classic')
from mpl_toolkits.basemap import Basemap
import numpy as np
from datetime import datetime
import math
import statistics as st

def OrnsteinUhlenbeckGenerator(Time,Speed,Lat,Lon,MAP,SPEED,MaxInterval,MinPeriod):
    #Time = Time vector in seconds
    #Speed = Speed vector in knots, raw from AIS data, filtered for speed records below steaming level
    #(Lat,Lon) = Coordinates
    #MAP, SEED = Plot dummies
    #MaxInterval = Maximum allowed interpolation interval
    #MinPeriod = Pre-defined time interval for resampling
    global TimeSerX
    global TimeSerY
    global TimeLat
    global TimeLon
    global RevRate1
    global MeanLev1
    global VOL1
    global X
    global IntSteps
    global IntTime
    global IntSpeed
    global Y

    AtTime = Time
    AtSpeed = Speed
    AtLat = Lat
    AtLon = Lon
    TimeSerX = list()                   #Preallocated transit time vector in unixtime
    TimeSerY = list()                   #Preallocated transit speed vector
    TimeLat = list()                    #Preallocated transit latitude vector
    TimeLon = list()                    #Preallocated transit longitude vector
    StartTime = list()                  #Preallocated list of transit start times
    EndTime = list()                    #Preallocated list of transit end times
    diffsort = list(np.diff(AtTime))    #Interval between speed records
    RevRate1 = list()                   #Preallocated list of mean reversion rates
    MeanLev1 = list()                   #Preallocated list of mean speed levels
    VOL1 = list()                       #Preallocated list of speed volatility
    N = 1000                            #Minimum number of records in a transit time series
    for x in range(0,len(AtTime)-1): #Loop through unmanipulated AIS data
        if diffsort[x] <= MaxInterval*3600: 
            #If yes, append speed records to transit time series
            TimeSerX.append(AtTime[x])      
            TimeSerY.append(AtSpeed[x])     
            TimeLat.append(AtLat[x])        
            TimeLon.append(AtLon[x])        
            
        elif diffsort[x] > MaxInterval*3600 and len(TimeSerX) > N: 
            #If yes, cut off time series and proceed to next criteria    
            if TimeSerX[-1] - TimeSerX[0] > MinPeriod*12*3600 and len(TimeSerX) > N: 
                #If yes, transit time series is qualified for data resampling
                StartTime.append(TimeSerX[0])
                EndTime.append(TimeSerX[-1])
                TimeFormat = 7200 # 1 = seconds, 60 = minutes, 3600 = hours
                TimeSpace = EndTime[-1] - StartTime[-1] 
                TimeInt = int((TimeSpace)/TimeFormat)
                SumSpeed = [0]*TimeInt
                TimeInc = [0]*TimeInt
                counttime = [0]*TimeInt

                #The following loops creates average speed samples
                for i in range(0,len(TimeSerX)): 
                    for j in range(0,TimeInt):
                        if TimeSerX[i] < TimeSerX[0] + (j+1)*TimeFormat and TimeSerX[i] >= TimeSerX[0] + j*TimeFormat:
                            SumSpeed[j] = SumSpeed[j]+TimeSerY[i]
                            TimeInc[j] = TimeSerX[0]+j*TimeFormat
                            counttime[j] = counttime[j]+1
                AvSpeed = [0]*TimeInt       
                for i in range(0,len(AvSpeed)):
                    if counttime[i] > 0:
                        AvSpeed[i] = SumSpeed[i]/counttime[i]
                    elif counttime[i] == 0:
                        AvSpeed[i] = 0
                #The following loops converts unixtime to datetime
                TimeInc = [x for x in TimeInc if x > 0]
                IncSteps = list()
                for i in range(0,len(TimeInc)):
                    Unixconv = datetime.fromtimestamp(TimeInc[i])
                    IncSteps.append(Unixconv)  
                    TimeSerXX = list()
                    AvSpeed = [x for x in AvSpeed if x > 0]
                for i in range(0,len(TimeSerX)):
                    Unixconv = datetime.fromtimestamp(TimeSerX[i])
                    TimeSerXX.append(Unixconv) 

                #The following loops allocates interpolated speed records where it is necessary
                IntTime = [0]*TimeInt
                for x in range(0,TimeInt):
                    IntTime[x]=min(TimeSerX) + x*TimeFormat
                IntSpeed = [0]*TimeInt

                for i in range(0,len(TimeInc)):
                    for j in range(0,len(IntTime)):
                        if TimeInc[i] == IntTime[j]:
                            IntSpeed[j] = AvSpeed[i] 

                for i in range(0,len(IntSpeed)):
                    count = 0
                    increments = 0
                    if IntSpeed[i] == 0:
                        count = 1
                        lower = IntSpeed[i-1]
                        for j in range(i+1,len(IntSpeed)):
                            if IntSpeed[j] == 0:
                                count += 1
                            elif IntSpeed[j] > 0:
                                upper = IntSpeed[j]
                                break
                        increments = (upper-lower)/(count+1)
                        for x in range(i,i+count):
                            IntSpeed[x] = increments*(1+x-i)+lower
                    elif IntSpeed[i] > 0:
                        count = 0
            
                IntSteps = list()
                for i in range(0,len(IntTime)):
                    Unixconv = datetime.fromtimestamp(IntTime[i])
                    IntSteps.append(Unixconv) 

                #The following part is the Ornstein-Uhlenbeck parameter estimation#
                #IntSpeed is the resampled data, including averaged and interpolated speed samples
                dx1 = np.diff(IntSpeed)
                dt = 2/24
                dxdt1 = dx1/dt
                del IntSpeed[-1]
                coeff1 = np.polyfit(IntSpeed,dxdt1,1)
                res1 = dxdt1 - np.polyval(coeff1,IntSpeed)
                revRate1  = -coeff1[0]
                meanLevel1 = st.mean(IntSpeed)
                vol1      = st.stdev(res1)*math.sqrt(dt)
                X = [0]*len(IntTime)
                Y = [0]*len(IntTime)
                Y[0]=IntSpeed[0]
                if revRate1 > 0.05:
                    for i in range(1,len(X)):
                        Y[i] = Y[i-1] + revRate1*(meanLevel1-Y[i-1])*dt + vol1*math.sqrt(dt)*np.random.normal(0,1)
                        #Y is the simulated speed process
                    if SPEED == 1:   
                        fig = plt.figure()
                        ax = fig.add_subplot(111)
                        ax.plot(TimeSerXX,TimeSerY,'k',label='AIS Data')
                        ax.plot(IntSteps,IntSpeed,c='k',label='Resampled AIS Data')
                        plt.plot(IntSteps,Y,'k--',label='Simulated Process')
                        plt.legend(loc='upper right')
                        plt.xlabel('Time')
                        plt.ylabel('Speed [knots]')
                        plt.show()
                    
                    if MAP == 1:
                        SpeedMap(TimeLat,TimeLon)
                        
                    RevRate1.append(revRate1)       #STORE ESTIMATED REVERSION RATE
                    MeanLev1.append(meanLevel1)     #STORE ESTIMATED MEAN LEVEL
                    VOL1.append(vol1)               #STORE ESTIMATED VOLATILITY
                    TimeSerX = list()
                    TimeSerXX = list()
                    TimeSerY = list()
                    TimeLat = list()
                    TimeLon = list()
                else:
                    TimeSerX = list()
                    TimeSerY = list()
                    TimeSerXX = list()
                    TimeLat = list()
                    TimeLon = list()                
            else:
                TimeSerX = list()
                TimeSerY = list()
                TimeSerXX = list()
                TimeLat = list()
                TimeLon = list()
        else:
            TimeSerX = list()
            TimeSerY = list()
            TimeSerXX = list()
            TimeLat = list()
            TimeLon = list()
    global SpeedData
    w, h = 3, len(VOL1);
    SpeedData1 = [[0 for x in range(w)] for y in range(h)]
    for i in range(0,h):
        SpeedData1[i][0] = MeanLev1[i]
        SpeedData1[i][1] = RevRate1[i]
        SpeedData1[i][2] = VOL1[i]

def SpeedMap(LAT,LON): #Plots the areas where transit time series are extracted from
    minlon = max(-180,min(LON)-10)
    minlat = max(-90,min(LAT)-10)
    maxlon = min(180,max (LON)+10)
    maxlat = min(90,max(LAT)+10)
    lat0 = (maxlat+minlat)/2
    lon0 = (maxlon+minlon)/2
    lat1 = (maxlat+minlat)/2-20
    fig=plt.figure()
    ax=fig.add_axes([0.1,0.1,0.8,0.8])
    m = Basemap(llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon,urcrnrlat=maxlat,\
            rsphere=(6378137.00,6356752.3142),\
            resolution='l',projection='merc',\
            lat_0=lat0,lon_0=lon0,lat_ts = lat1)
    ax.annotate('Start', xy=(LAT[0], LON[0]), xytext=(TimeLon[0], TimeLat[0]+2),
                                arrowprops=dict(facecolor='black', shrink=0.05),)
    m.drawcoastlines()
    m.fillcontinents()
    x, y = m(LON,LAT)
    m.scatter(x,y,0.1,marker='o',c='k')
    m.drawparallels(np.arange(-90,90,20),labels=[1,1,0,1])       
    m.drawmeridians(np.arange(-180,180,30),labels=[1,1,0,1])

if __name__ == "__main__":    
    OrnsteinUhlenbeckGenerator()
    SpeedMap()