#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Nov 18 17:34:45 2016

@author: jonleonhardsen
"""

import sqlite3
import matplotlib.pyplot as plt


from netCDF4 import Dataset, num2date
import time
import datetime
from mpl_toolkits.basemap import Basemap
import urllib, os
import numpy as np
from scipy import stats
import statistics as st
import pandas


# create new figure, axes instances


def ExtractData(filepath,a,b,c,d):
    global draughttime
    global useridDraught
    global draught
    global plotlat
    global plotlon
    global speeds
    global geospeed
    global valspeed
    global timestep
    
    speeds = list()
    plotlat = list()
    plotlon = list()
    valspeed = list()
    lats = list()
    lons = list()
    unistep = list()
    timestep = list()
    geospeed = list()

    conn = sqlite3.connect(filepath)
    cur = conn.cursor()
    SegmentAnalysis = 1
    with conn:
        cur = conn.cursor()  
        if SegmentAnalysis == 1:
            cur.execute("select unixtime,sog,latitude,longitude,userid from PanamaxSample1 order by userid, unixtime asc")
        else:
            cur.execute("SELECT unixtime,sog,latitude,longitude,userid FROM MessageType1 ORDER BY UNIXTIME ASC")
        VesselData = cur.fetchall()
        
        maxLon = a
        maxLat = b
        minLon = c
        minLat = d
        lowtime = 142007040
        hightime = 14515200000
        
        #####################################################
        # THE FOLLOWING PARTS FILTERS ON SPEED AND POSITION #
        #####################################################
        for i in range(0,len(VesselData)):
            Datastrip = VesselData[i]
            Speed = Datastrip[1]
            untime = Datastrip[0]
            lat = Datastrip[2]
            lon = Datastrip[3]
            if Speed < 25 and Speed > 0 and lat <= maxLat and lat >= minLat \
                and lon >= minLon and lon <= maxLon and untime > lowtime and untime < hightime:     
                speeds.append(Speed)
                lats.append(lat)
                lons.append(lon)
                unistep.append(untime)    
                timestep.append(untime)
                geospeed.append(Speed)
                plotlat.append(lat)
                plotlon.append(lon)
    
    cur.close()
    conn = sqlite3.connect(filepath)
    cur = conn.cursor()
    with conn:
        cur = conn.cursor()
        if SegmentAnalysis == 1:
            cur.execute("select unixtime,draught,userid from Panamax5 order by userid, unixtime asc")
        else:
            cur.execute("SELECT unixtime,draught,userid FROM MessageType5 ORDER BY UNIXTIME ASC")
        draughtdata = cur.fetchall()
        draughttime = list()
        draught = list()
        useridDraught = list()
        
    for i in range(0,len(draughtdata)):
        draughtstrip = draughtdata[i]
        if Speed < 30 and Speed > 0 and lat <= maxLat and lat >= minLat and lon >= minLon and lon <= maxLon and untime > lowtime and untime < hightime:
            if draughtstrip[1]/10 > 5:
                draughttime.append(draughtstrip[0])
                draught.append(draughtstrip[1]/10)
                useridDraught.append(draughtstrip[2])

####################################################
# THE FOLLOWING PART PLOTS HISTOGRAM OF SPEED DIST #
####################################################




from datetime import datetime
def GeoDistribution():
    global PacLat
    global PacLon
    global PacSpeed
    global AtlLat
    global AtlLon
    global AtlSpeed
    global SuezLat
    global SuezLon
    global SuezSpeed
    global PacMean
    global AtlMean
    global SuezMean
    global PacVol
    global AtlVol
    global SuezVol
    
    PacLat = list()
    PacLon = list()
    AtlLat = list()
    AtlLon = list()
    SuezLat = list()
    SuezLon = list()
    PacSpeed = list()
    AtlSpeed = list()
    SuezSpeed = list()
    
    for i in range(0,len(plotlat)):
        if plotlat[i] > 30 and (plotlon[i]>=150 or plotlon[i] <= -120):
            PacLat.append(plotlat[i])
            PacLon.append(plotlon[i])
            PacSpeed.append(speeds[i])
        elif plotlat[i]>10 and plotlon[i]>=-75 and plotlon[i]<= -10:
            AtlLat.append(plotlat[i])
            AtlLon.append(plotlon[i])
            AtlSpeed.append(speeds[i])
        elif plotlat[i] >= 0 and plotlat[i]<= 30 and plotlon[i] >=30 and plotlon[i] <= 100:
            SuezLat.append(plotlat[i])
            SuezLon.append(plotlon[i])
            SuezSpeed.append(speeds[i])
            
    PacMean = st.mean(PacSpeed)
    PacVol = st.stdev(PacSpeed)
    AtlMean = st.mean(AtlSpeed)
    AtlVol = st.stdev(AtlSpeed)
    SuezMean = st.mean(SuezSpeed)
    SuezVol = st.stdev(SuezSpeed)

def SpeedHistogram():
    plt.figure()
    hist, bins = np.histogram(speeds, bins=50)
    width = 0.7 * (bins[1] - bins[0])
    center = (bins[:-1] + bins[1:]) / 2
    plt.bar(center, hist, align='center', width=width)
    plt.show()

    plt.figure()
    histval1, binsval = np.histogram(speeds, bins=50)
    histval = histval1/sum(histval1)
    width = 0.7 * (binsval[1] - binsval[0])
    center = (binsval[:-1] + binsval[1:]) / 2
    plt.bar(center, histval, align='center', width=width,color='k')
    plt.xlabel('Speed [knots]')
    plt.ylabel('Fraction of time')
    plt.show()

    
    plt.figure()
    histval1, binsval = np.histogram(draught, bins=20)
    histval = histval1/sum(histval1)
    width = 0.7 * (binsval[1] - binsval[0])
    center = (binsval[:-1] + binsval[1:]) / 2
    plt.bar(center, histval, align='center', width=width,color='k')
    plt.xlabel('Draught [meters]')
    plt.ylabel('Fraction of time')
    plt.show()
################################################
## THE FOLLOWING PARTS PLOTS POSITIONS ON MAPS #
################################################
def GlobalMap():
    plt.figure()
    lats = plotlat
    lons = plotlon
    m = Basemap(projection='hammer',lon_0=0)
    x, y = m(lons,lats)
    m.drawmapboundary(fill_color='#99ffff')
    #m.fillcontinents(color='#cc9966',lake_color='#99ffff')
    m.scatter(x,y,10,marker='o',c='c',alpha=0.8)
    m.bluemarble()
    plt.show()

#########################################################
def LocalMap():
    minlon = max(-180,min(plotlon)-10)
    minlat = max(-90,min(plotlat)-10)
    maxlon = min(180,max (plotlon)+10)
    maxlat = min(90,max(plotlat)+10)
    lat0 = (maxlat+minlat)/2
    lon0 = (maxlon+minlon)/2
    lat1 = (maxlat+minlat)/2-20

    fig=plt.figure()
    ax=fig.add_axes([0.1,0.1,0.8,0.8])
    m = Basemap(llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon,urcrnrlat=maxlat,\
            rsphere=(6378137.00,6356752.3142),\
            resolution='l',projection='cyl',\
            lat_0=lat0,lon_0=lon0,lat_ts = lat1)

    m.drawcoastlines()
    m.fillcontinents()
    x, y = m(plotlon,plotlat)
    m.scatter(x,y,0.1,marker='o',c='k')
    m.drawparallels(np.arange(-90,90,20),labels=[1,1,0,1])       
    m.drawmeridians(np.arange(-180,180,20),labels=[1,1,0,1])
    #m.bluemarble()
    
       
   
################################################
## THE FOLLOWING PART PLOTS SPEED AGAINST TIME #
################################################

def SpeedDev(): 
    Timesteps = list()
    for i in range(0,len(timestep)):
        unixconv = datetime.fromtimestamp(timestep[i])
        Timesteps.append(unixconv)
 
    plt.figure()
    plt.plot(Timesteps,geospeed,'k')
    plt.show()
    plt.ylabel("Speed [knots]")   
    
def DraughtInterpolate():    
    global IntDraught    
    IntDraught = list()
    draughtSpeed = list()
    draughtIntTime = list()
    for i in range(0,len(timestep)):
        
        if timestep[i] >= draughttime[0]:
            idx = (np.abs(array(draughttime)-timestep[i])).argmin()
            IntDraught.append(draught[idx])
            draughtSpeed.append(speeds[i])
            draughtIntTime.append(timestep[i])
            print(i)
    
    speeds = 5
    draughts = 3
    operProf = np.zeros((draughts,speeds))
    mindraught = min(IntDraught)-0.01
    maxdraught = max(IntDraught)
    minspeed = 12
    maxspeed = max(draughtSpeed)
    deltaSpeed = maxspeed-minspeed
    deltaDraught = maxdraught-mindraught
    speedInt = deltaSpeed/speeds
    draughtInt = deltaDraught/draughts
    for i in range(0,len(IntDraught)):
        print(i)
        for a in range(0,draughts):
            for b in range(0,speeds):
                if draughtSpeed[i] > minspeed + b*speedInt and draughtSpeed[i] <= minspeed + (b+1)*speedInt and IntDraught[i] > mindraught + a*draughtInt and IntDraught[i] <= mindraught + (a+1)*draughtInt:
                    operProf[a,b] += 1
                    #print(a)
                    #print(b)
                
    
    sumOp = sum(operProf)
    normOp = np.zeros((draughts,speeds))               
    for a in range(0,draughts):
        for b in range(0,speeds):
            normOp[a,b] = operProf[a,b]/sumOp
                       
                    
def TradeDirectionSuez():
    global eastBoundlonS
    global westBoundlonS
    global eastBoundlatS
    global westBoundlatS
    global eastBoundSpeedS
    global westBoundSpeedS
    global eastBoundDraughtS
    global westBoundDraughtS
    global eastBoundTimeS
    global westBoundTimeS
    eastBoundlonS = list()
    westBoundlonS = list()
    eastBoundlatS = list()
    westBoundlatS = list()
    eastBoundSpeedS = list()
    westBoundSpeedS = list()
    eastBoundDraughtS = list()
    eastBoundTimeS = list()
    westBoundTimeS = list()
    
    
    for i in range(1,len(plotlat)):
        if plotlon[i] < 0 and plotlon[i] > - 11 and plotlat[i] > 40:
            if plotlat[i] < plotlat[i-1]:
                eastBoundlonS.append(plotlon[i])
                eastBoundlatS.append(plotlat[i])
                eastBoundSpeedS.append(speeds[i])
                eastBoundTimeS.append(timestep[i])
                #eastBoundDraught.append(dra)
            else:
                westBoundlonS.append(plotlon[i])
                westBoundlatS.append(plotlat[i])
                westBoundSpeedS.append(speeds[i])
                westBoundTimeS.append(timestep[i])
        elif plotlat[i] <= 40 and plotlon[i] < -7.5 and plotlon[i] >= -11 and plotlat[i] >= 35:
            
            if plotlat[i] < plotlat[i-1]:
                eastBoundlonS.append(plotlon[i])
                eastBoundlatS.append(plotlat[i])
                eastBoundSpeedS.append(speeds[i])
                eastBoundTimeS.append(timestep[i])
            else:
                westBoundlonS.append(plotlon[i])
                westBoundlatS.append(plotlat[i])
                westBoundSpeedS.append(speeds[i])
                westBoundTimeS.append(timestep[i])
        elif plotlon[i] >= -7.5 and plotlat[i] < 45 and plotlat[i] >= 0 and plotlon[i] <= 110:
            if plotlon[i] > plotlon[i-1]:
                eastBoundlonS.append(plotlon[i])
                eastBoundlatS.append(plotlat[i])
                eastBoundSpeedS.append(speeds[i])
                eastBoundTimeS.append(timestep[i])
            else:
                westBoundlonS.append(plotlon[i])
                westBoundlatS.append(plotlat[i])
                westBoundSpeedS.append(speeds[i])
                westBoundTimeS.append(timestep[i])
                

    minlon = max(-180,min(westBoundlonS)-10)
    minlat = max(-90,min(westBoundlatS)-10)
    maxlon = min(180,max (westBoundlonS)+10)
    maxlat = min(90,max(westBoundlatS)+10)
    lat0 = (maxlat+minlat)/2
    lon0 = (maxlon+minlon)/2
    lat1 = (maxlat+minlat)/2-20

    fig=plt.figure()
    ax=fig.add_axes([0.1,0.1,0.8,0.8])
    m = Basemap(llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon,urcrnrlat=maxlat,\
            rsphere=(6378137.00,6356752.3142),\
            resolution='l',projection='cyl',\
            lat_0=lat0,lon_0=lon0,lat_ts = lat1)

    m.drawcoastlines()
    #m.fillcontinents()
    x, y = m(eastBoundlonS,eastBoundlatS)
    X, Y = m(westBoundlonS,westBoundlatS)
    m.scatter(x,y,10,marker='o',c='k',alpha = 1)
    m.scatter(X,Y,10,marker='D',c='w',alpha = 0.1)
    m.drawparallels(np.arange(-90,90,20),labels=[1,1,0,1])       
    m.drawmeridians(np.arange(-180,180,20),labels=[1,1,0,1])
    m.bluemarble()
#    #m.bluemarble()

    plt.figure()
    bins = np.linspace(0, 25, 51)
    plt.hist(westBoundSpeedS, bins, alpha=0.6, label='West Bound',color='k',normed=True)
    plt.hist(eastBoundSpeedS, bins, alpha=0.6, label='East Bound',color='w',normed=True)
    plt.legend(loc='upper right')
    plt.xlabel('Speed [knots]')
    plt.ylabel('Fraction of time')
    plt.show()
                
def FreightRateImpact():
    global highRateSpeed
    global medRateSpeed
    global lowRateSpeed
    
    highRateSpeed = list()
    medRateSpeed = list()
    lowRateSpeed = list()
    
    for i in range(0,len(speeds)):
        if timestep[i] > 1293840000 and timestep[i] <= 1309392000:
            highRateSpeed.append(speeds[i])
        elif timestep[i] > 1388534400 and timestep[i] <= 1404086400:
            lowRateSpeed.append(speeds[i])
        elif timestep[i] > 1422748800 and timestep[i] <= 1438300800:
            medRateSpeed.append(speeds[i])
    
    
def TradeDirectionPacific():
    global eastBoundlonP
    global westBoundlonP
    global eastBoundlatP
    global westBoundlatP
    global eastBoundSpeedP
    global westBoundSpeedP
    eastBoundlonP = list()
    westBoundlonP = list()
    eastBoundlatP = list()
    westBoundlatP = list()
    eastBoundSpeedP = list()
    westBoundSpeedP = list()
    
    for i in range(1,len(plotlat)):
        if (plotlon[i] > 140 or plotlon[i] < -130) and plotlat[i] > 30:
            if plotlon[i] > plotlon[i-1]:
                eastBoundlonP.append(plotlon[i])
                eastBoundlatP.append(plotlat[i])
                eastBoundSpeedP.append(speeds[i])
            elif plotlon[i] < -160 and plotlon[i-1] > 170:
                eastBoundlonP.append(plotlon[i])
                eastBoundlatP.append(plotlat[i])
                eastBoundSpeedP.append(speeds[i])
            elif plotlon[i] < plotlon[i-1]:
                westBoundlonP.append(plotlon[i])
                westBoundlatP.append(plotlat[i])
                westBoundSpeedP.append(speeds[i])
            elif plotlon[i] > -160 and plotlon[i-1] < 170:
                westBoundlonP.append(plotlon[i])
                westBoundlatP.append(plotlat[i])
                westBoundSpeedP.append(speeds[i])

    pacificSpeeds = westBoundSpeedP+eastBoundSpeedP
    std = st.stdev(pacificSpeeds)
    mean = st.mean(pacificSpeeds)
    plt.figure()
    bins = np.linspace(0, 25, 51)
    plt.hist(westBoundSpeedP+eastBoundSpeedP, bins, alpha=0.6, label='December',color='k',normed=True)
    plt.legend(loc='upper right')
    plt.xlabel('Speed [knots]')
    plt.ylabel('Fraction of time')
    plt.title('Speed mean = %s, Speed std = %s'%(mean,std))
    plt.show()       

def MonthlyAverage():
    global MarketRates
    MarketRates = pandas.read_excel('/Users/jonleonhardsen/Documents/Documents/Skole/NTNU vår 2017/RatesOilPrice.xlsx')
    global monthlyMeanWS
    global monthlyMeanES
    global monthlyMean
    global monthlyVol
    global months
    global freightRates
    global oilPrice
    global sortedtimeWS
    global sortedtimeES
    global countMess
    global monthsCon
    august2010 = 1280620800
    december2015 = 1451606354
    Intervals = 5+5*12
    Increments = round((december2015-august2010)/Intervals)
    monthlyMeanWS = list()
    monthlyMeanES = list()
    monthlyMean = list()
    monthlyVol = list()
    months = list()
    freightRates = list()
    oilPrice = list()
    countMess = list()
    marketData = MarketRates.as_matrix()
    sortedtimeWS, sortedspeedWS = zip(*sorted(zip(westBoundTimeS, westBoundSpeedS)))
    sortedtimeES, sortedspeedES = zip(*sorted(zip(eastBoundTimeS, eastBoundSpeedS)))
    sortedtime, sortedspeed = zip(*sorted(zip(timestep, geospeed)))
    
    count = 1
    countW = 1
    countE = 1
    for i in range(0,Intervals-1):
        print(i)
        monthlyspeedWS = list()
        monthlyspeedES = list()
        monthlyspeed = list()

        j = countW         
        while sortedtimeWS[j] <= august2010 + Increments*i and sortedtimeWS[j] > august2010 + Increments*(i-1):
                monthlyspeedWS.append(sortedspeedWS[j])
                j += 1
        countW = j+1
        
        if len(monthlyspeedWS)>1:        
            monthlyMeanWS.append(st.mean(monthlyspeedWS))
        else:
            monthlyMeanWS.append(0)
       
        j = countE    
        while sortedtimeES[j] <= august2010 + Increments*i and sortedtimeES[j] > august2010 + Increments*(i-1):
                monthlyspeedES.append(sortedspeedES[j])
                j += 1
        countE = j+1
        
        if len(monthlyspeedES)>1:
            monthlyMeanES.append(st.mean(monthlyspeedES))
        else:
            monthlyMeanES.append(0)
        
        j = count   
        while sortedtime[j] <= august2010 + Increments*i and sortedtime[j] > august2010 + Increments*(i-1):
                monthlyspeed.append(sortedspeed[j])
                j += 1
        count = j+1
        
        if len(monthlyspeed)>1:
            monthlyMean.append(st.mean(monthlyspeed))
        else:
            monthlyMean.append(0)
        
        countMess.append(len(monthlyspeed))
        months.append(august2010+Increments*i)
        monthsCon = list()
        for i in range(0,len(months)):
            unixconv = datetime.fromtimestamp(months[i])
            monthsCon.append(unixconv)
        
        freightRates.append(marketData[i][1])
        oilPrice.append(marketData[i][2])
        
    plt.figure()
    plt.bar(monthsCon,monthlyMeanWS,alpha = 0.8, color='k',label='West Bound Suez')
    plt.bar(monthsCon,monthlyMeanES,alpha = 0.8, color='w',label='East Bound Suez')
    plt.xlabel('Months')
    plt.ylabel('Speed [knots]')
    plt.legend(loc='upper right')
    
    plt.figure()
    plt.bar(monthsCon,monthlyMean,alpha = 0.8, color='k',label='Global')
    
    plt.xlabel('Months')
    plt.ylabel('Speed [knots]')
    plt.legend(loc='upper right')
    
    
    years = [1293796800.0, 1325332800.0, 1356868800.0, 1388404800.0, 1419940800.0, 1451476800.0]
    yearlyAv = list()
    yearSpeed = list()
    for i in range(1,len(years)):
        yearSpeed = list()
        for j in range(0,len(monthlyMean)):
            if months[j] <= years[i]:
                yearSpeed.append(monthlyMean[j])
        
        yearlyAv.append(st.mean(yearSpeed))
    
    yearsCon = list()
    del years[-1]
    for i in range(0,len(years)):
        unixconv = datetime.fromtimestamp(years[i])
        yearsCon.append(unixconv)
   


############################################################################
## THE FOLLOWING PART CALCULATES INTERVALS OF MESSAGES AND PLOTS HISTOGRAM #
############################################################################
def SpeedForAnalysis():
    global logdiff
    global UpDiff
    global UpTime
    global UpSpeed
    global AnSpeed
    global AnTime
    global AnSteps
    global AnDiff
    global AnLat
    global AnLon
    logdiff = list()
    logdiff = np.diff(timestep)  
    UpDiff = list()
    UpSpeed = speeds
    UpTime = timestep
    UpDiff = logdiff
    UpLat = plotlat
    UpLon = plotlon
    
    for i in range(0,len(logdiff)):
        if logdiff[i] < 1:
            logdiff[i] = logdiff[i]
            UpSpeed[i+1] = 101010
            UpTime[i+1] = 101010
            UpDiff[i] = 101010

    AnSpeed = [x for x in UpSpeed if x != 101010] #AnDiff = Message intervals for analysis
    AnTime = [x for x in UpTime if x != 101010]
    AnLat = [x for x in UpLat if x != 101010]
    AnLon = [x for x in UpLon if x != 101010] #AnSpeed = Speed for analysis
    AnDiff = [x for x in UpDiff if x != 101010] #AnTime = Time for analysis
    AnDiff.append(1)
    AnSteps = list()
    for i in range(0,len(AnTime)):
        Unixconv = datetime.fromtimestamp(AnTime[i])
        AnSteps.append(Unixconv)


################################################
## THE FOLLOWING PART PLOTS INTERVAL FREQUENCY #
################################################
def IntervalFreq():    
    Count = list()
    Number = list()
    for i in range(0,360):
        Count.append(i)
        Number.append(AnDiff.count(i))
    
    plt.figure()
    plt.plot(Count,Number)
    plt.show()   
    plt.figure()
    plt.scatter(AnSteps,AnSpeed,c='k')
    #plt.plot(IncSteps,AvSpeed,'k')
    plt.show()
    
    
def GeoAnalysis(max_lon,max_lat,min_lon,min_lat,minspeed):
    global AtSpeed
    global AtTime
    global AtLat
    global AtLon
    global AtSteps
    
    AtMaxLon = max_lon
    AtMaxLat = max_lat
    AtMinLon = min_lon
    AtMinLat = min_lat
    AtSpeed = list()
    AtTime = list()
    AtLat = list()
    AtLon = list()

    for i in range(0,len(AnSpeed)):
        #print(i)
        if AnSpeed[i] > minspeed and AnLat[i] > AtMinLat and AnLat[i] < AtMaxLat and (AnLon[i] > AtMinLon or AnLon[i] < AtMaxLon):
            AtSpeed.append(AnSpeed[i])
            AtTime.append(AnTime[i])
            AtLat.append(AnLat[i])
            AtLon.append(AnLon[i])
        
    AtSteps = list()
    for i in range(0,len(AtTime)):
        Unixconv = datetime.fromtimestamp(AtTime[i])
        AtSteps.append(Unixconv)


def DraughtAnalysis():    
    global AnDraught
    global AnCat
    global DSpeed
    global DTime    
    global Dcat
    global slope
    global intercept
    global r_value
    global p_value
    global std_err
    Drange = max(draught)-min(draught)
    Intervals = 3
    Dsteps = Drange/Intervals
    Dcat = [0]*len(draught)

    for i in range(0,len(draught)):
        for j in range(0,Intervals-1):
            if  min(draught) + Dsteps*j <= draught[i] <= min(draught)+(j+1)*Dsteps :
                Dcat[i] = j+1

    AnDraught = list()
    AnCat = list() 
    DSpeed = list()
    DTime = list()
    for j in range(0,len(Dcat)-1):
        for i in range(0,len(AnTime)):
        #if AnTime[i]
            if AnTime[i]>=draughttime[j] and AnTime[i]<draughttime[j+1]:
                AnDraught.append(draught[j])
                AnCat.append(Dcat[j])
                DSpeed.append(AnSpeed[i])
                DTime.append(AnTime[i])
    slope, intercept, r_value, p_value, std_err = stats.linregress(AnDraught,DSpeed) 

    
def CumulatedSpeed():
    import math
    global SRange
    global SortSpeed
    global SpeedDist
    global CumSpeed
    global NSpeedDist
    global NCumSpeed
    global InterSpeed
    SRange = math.ceil(max(AnSpeed))
    SortSpeed = np.sort(AnSpeed)
    SpeedDist = [0]*SRange
    CumSpeed = [0]*SRange
    InterSpeed = [0]*SRange
    for i in range(0,len(AnSpeed)):
        for j in range(1,SRange+1):
            InterSpeed[j-1] = j
            if SortSpeed[i] < j:
                CumSpeed[j-1] = CumSpeed[j-1] + 1
                if SortSpeed[i] >= j-1:
                    SpeedDist[j-1] = SpeedDist[j-1] + 1

    NSpeedDist = [0]*SRange
    NCumSpeed = [0]*SRange
    for i in range(0,SRange):
        NSpeedDist[i] = SpeedDist[i]/len(AnSpeed)
        NCumSpeed[i] = CumSpeed[i]/len(AnSpeed)
        

if __name__ == "__main__":       
    ExtractData(filepath,a,b,c,d)#'/Volumes/LaCie/SAVONA.db')
    GlobalMap()
    LocalMap()
    SpeedDev()
    SpeedHistogram()
    SpeedForAnalysis()
    IntervalFreq()
    GeoAnalysis()
    DraughtAnalysis()
    CumulatedSpeed()

